FILES = [
    ".gitignore",
    ".gitattributes",
    ".gitmodules",
    ".git",
    "LICENSE",
    ".python-version",
    "uv.lock",
    "poetry.lock",
    ".dockerignore",
    ".coverage",
    ".pre-commit-config.yaml",
]

EXTENSIONS = [
    "*.png",
    "*.jpg",
    "*.jpeg",
    "*.gif",
    "*.svg",
    "*.ico",
    "*.woff",
    "*.woff2",
    "*.ttf",
    "*.eot",
    "*.otf",
    "__pycache__",
    "*.pyc",
    "*.pyo",
    "*.pyd",
    ".DS_Store",
    "*.zip",
    "*.far",
    "*.fst",
    "*.tsv",
    "*.csv",
]

PATTERNS = [
    ".git",
    ".venv",
    ".vscode",
    ".idea",
    "node_modules",
    "build",
    "dist",
    "target",
    ".vs",
    "bin",
    "obj",
    "publish",
    "tests",
    "test",
    ".ruff_cache",
]
